/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.createVar;
import static org.fortiss.af3.expression.utils.ExpressionModelElementFactory.funcCall;
import static org.fortiss.tooling.base.utils.AnnotationUtils.getAnnotation;

import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.expression.model.terms.IExpressionTerm;
import org.fortiss.af3.platform.language.executable.TransmitterExecutableBase;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.ITermExecutable;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.IWriteExecutableWithNoValSupport;
import org.fortiss.af3.platform.raspberry.generator.executable.interfaces.InitExecutable;
import org.fortiss.af3.platform.raspberry.model.ActuatorOut;
import org.fortiss.af3.platform.raspberry.model.annotation.PinNumber;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class ActuatorOutExecutable extends TransmitterExecutableBase<ActuatorOut> implements
		InitExecutable, ITermExecutable, IWriteExecutableWithNoValSupport {

	/**
	 * @param modelElement
	 */
	public ActuatorOutExecutable(ActuatorOut modelElement) {
		super(modelElement);
		// TODO Auto-generated constructor stub
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization() {
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("init_Actuator", createVar(actuatorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination() {
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("term_Actuator", createVar(actuatorId));
	}

	/** {@inheritDoc} */
	@Override
	/* generates call: write_Actuator(pinNr, varName) */
	public IExpressionTerm getValueWriteAccessor(OutputPort logicalSignal, IExpressionTerm value) {
		PinNumber ias = getAnnotation(modelElement, PinNumber.class);
		int pinNr = ias.getPinNumber();
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("write_Actuator", createVar(pinNr + ""), createVar(actuatorId));

	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getNoValWriteAccessor(OutputPort logicalSignal) {
		return funcCall("set_noval_Actuator");
	}

	/** {@inheritDoc} */
	@Override
	/* generates call: write_Actuator(pinNr, varName) */
	public IExpressionTerm getValueWriteAccessor(OutputPort logicalSignal, IExpressionTerm value,
			Deployment depl) {
		PinNumber ias = getAnnotation(modelElement, PinNumber.class);
		int pinNr = ias.getPinNumber();
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("write_Actuator", createVar(pinNr + ""), createVar(actuatorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(IExpressionTerm signal) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getTermination(Deployment depl) {
		// TODO Auto-generated method stub
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("term_Actuator", createVar(actuatorId));
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm term) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(IExpressionTerm port, IExpressionTerm portName,
			IExpressionTerm readOrWrite) {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExpressionTerm getInitialization(Deployment depl) {
		// TODO Auto-generated method stub
		String actuatorId = modelElement.getName() + "_ID_" + modelElement.getId();
		return funcCall("init_Actuator", createVar(actuatorId));
	}

}
