/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.platform.raspberry.generator.executable;

import static org.fortiss.af3.generator.common.utils.SourceModelElementFactory.createStaticContentSourceUnit;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryUtils.getEcuForComponent;
import static org.fortiss.af3.platform.raspberry.utils.RaspberryUtils.getTargetComponent;
import static org.fortiss.tooling.kernel.utils.KernelModelElementUtils.findElementById;

import java.util.List;
import java.util.Map;

import org.fortiss.af3.bounds.model.VariableBoundsSpecification;
import org.fortiss.af3.component.model.Component;
import org.fortiss.af3.component.model.OutputPort;
import org.fortiss.af3.component.model.Port;
import org.fortiss.af3.component.model.PortSpecification;
import org.fortiss.af3.deployment.model.ComponentAllocation;
import org.fortiss.af3.deployment.model.Deployment;
import org.fortiss.af3.generator.common.model.source.SourcePackage;
import org.fortiss.af3.platform.language.executable.TransmissionUnitExecutableBase;
import org.fortiss.af3.platform.model.ExecutionUnit;
import org.fortiss.af3.platform.model.TransmissionUnit;
import org.fortiss.af3.platform.raspberry.generated.files.RaspberryPortTypeGeneration;

/**
 * 
 * @author chaudhary
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class BusExecutable extends TransmissionUnitExecutableBase<TransmissionUnit> {

	/**
	 * @param modelElement
	 */
	public BusExecutable(TransmissionUnit modelElement) {
		super(modelElement);
		// TODO Auto-generated constructor stub
	}

	/** {@inheritDoc} */
	@Override
	public void createAllFiles(SourcePackage pack, Map<ExecutionUnit, List<Port>> euPortsPair,
			Map<ExecutionUnit, List<Component>> euComponentList, Component topComponent,
			int deploymentID) {
		findBusSignals(deploymentID, topComponent, pack);
	}

	/** Finds Information about Signals that are transmitted over the Bus. */
	public void findBusSignals(int deploymentID, Component topComponent, SourcePackage pack) {
		String resultString = "";
		Deployment currentDeployment =
				(Deployment)findElementById(deploymentID, topComponent.eContainer().eContainer());
		for(ComponentAllocation allocation : currentDeployment.getComponentAllocations()) {
			Component c1 = allocation.getComponent();
			for(Object port : c1.getConnectors()) {
				if(port instanceof OutputPort) {
					for(Component c2 : getTargetComponent((OutputPort)port)) {
						if(getEcuForComponent(c1, currentDeployment) != getEcuForComponent(c2,
								currentDeployment) &&
								getEcuForComponent(c2, currentDeployment) != null) {
							resultString += "Name: " + ((OutputPort)port).getName() + ", ";
							for(Object specs : ((OutputPort)port).getSpecifications()) {
								if(specs instanceof PortSpecification) {
									PortSpecification portSpec = (PortSpecification)specs;
									resultString += "Type: " + portSpec.getType() + ", ";
								}
								if(specs instanceof VariableBoundsSpecification) {
									VariableBoundsSpecification varBoundSpec =
											(VariableBoundsSpecification)specs;
									resultString +=
											"From: " + varBoundSpec.getLowerBound() + " To: " +
													varBoundSpec.getUpperBound() + " ";
								}
							}
							resultString += "\n";
						}
					}
				}
			}

		}

		RaspberryPortTypeGeneration rspPortInfoGen = new RaspberryPortTypeGeneration(resultString);
		rspPortInfoGen.createInfofile();
		pack.addUnit(createStaticContentSourceUnit("PortSpecifications",
				rspPortInfoGen.getContent(), false));
	}
}
